/* @(#)97	1.18  com/inc/sys/msg.h, bos, bos320 6/16/90 00:32:41 */
#ifndef _H_MSG
#define _H_MSG

/*
 * COMPONENT_NAME: (SYSIPC) IPC Message Facility
 *
 * FUNCTIONS:
 *
 * ORIGINS: 27 3
 *
 * (C) COPYRIGHT International Business Machines Corp. 1987, 1989
 * All Rights Reserved
 * Licensed Materials - Property of IBM
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 */

#include <standards.h>
#include <sys/ipc.h>

#ifdef _XOPEN_SOURCE

/*
 *	Message Operation Flags.
 */

#define	MSG_NOERROR	010000	/* no error if big message */

/*
 *	Structure Definitions
 */

/*
 *	There is one msg queue id data structure for each q in the system.
 */

struct msqid_ds {
	struct ipc_perm	msg_perm;	/* operation permission struct */
	struct msg	*msg_first;	/* ptr to first message on q */
	struct msg	*msg_last;	/* ptr to last message on q */
	unsigned short	msg_cbytes;	/* current # bytes on q */
	unsigned short	msg_qnum;	/* # of messages on q */
	unsigned short	msg_qbytes;	/* max # of bytes on q */
	pid_t		msg_lspid;	/* pid of last msgsnd */
	pid_t		msg_lrpid;	/* pid of last msgrcv */
	time_t		msg_stime;	/* last msgsnd time */
	time_t		msg_rtime;	/* last msgrcv time */
	time_t		msg_ctime;	/* last change time */
	/* event list of messages for this queue */
	int		msg_rwait;	/* wait list for message receive */
	int		msg_wwait;	/* wait list for message send */
/*      The following members are in support of msgselect() */
	unsigned short	msg_reqevents;	/* select/poll requested events */
};

#ifdef _NO_PROTO
extern int msgctl();
extern int msgget();
extern int msgrcv();
extern int msgsnd();
#else
extern int msgget(key_t key, int msgflg);
extern int msgrcv(int msqid, void *msgp, size_t msgsz, long msgtyp, int msgflg);
extern int msgsnd(int msqid, void *msgp, size_t msgsz, int msgflg);
extern int msgctl(int msqid, int cmd, struct msqid_ds *buf);
#endif /* _NO_PROTO */

#endif /* _XOPEN_SOURCE */

#ifdef	_ALL_SOURCE
/*
 *	ipc_perm Mode Definitions.
 */
#define	MSG_R		IPC_R	/* read permission */
#define	MSG_W		IPC_W	/* write permission */
#define	MSG_RWAIT	01000	/* a reader is waiting for a message */
#define	MSG_WWAIT	02000	/* a writer is waiting to send */

/*
 *	Message header information
 */

#define	MSGX	time_t	mtime;		/* time message was sent */	\
		uid_t	muid;		/* author's effective uid */	\
		gid_t	mgid;		/* author's effective gid */	\
		pid_t	mpid;		/* author's process id */	\

struct msg_hdr	{ 
		 MSGX
		 mtyp_t	mtype;		/* message type */
};

/*
 *	There is one msg structure for each message that may be in the system.
 */

struct msg {
	struct msg     *msg_next;	/* ptr to next message on q */
	struct msg_hdr 	msg_attr;	/* message attributes */
	unsigned short	msg_ts; 	/* message text size */
	char		*msg_spot;	/* pointer to message text */
};


/*
 *	User message buffer template for msgsnd and msgrcv system calls.
 */

struct msgbuf {
	mtyp_t	mtype;		/* message type */
	char	mtext[1];	/* message text */
};

/*
 *	User message buffer template for msgxrcv system call.
 */

struct msgxbuf {
		MSGX
		mtyp_t	mtype;	  /* Message type */
		char	mtext[1]; /* Message text */
};

/* We need to know the length of everything but mtext[1] and padding. */
#define MSGXBUFSIZE (int)(((struct msgxbuf *)NULL)->mtext)



/* rmsgsnd() and rmsgrcv() "flags" parameter definitions */
#define MSG_SYSSPACE	0x01
#define XMSG 		0x02

/*
 *	Message information structure.
 */

struct msginfo {
	int	msgmax,	/* max message size			*/
		msgmnb,	/* max # bytes on queue 		*/
		msgmni,	/* # of message queue identifiers	*/
		msgmnm;	/* max # messages per queue identifier	*/
};

/*
 * Number of bytes to copy for IPC_STAT command
 */
#define MSG_STAT  (int)&(((struct msqid_ds *)NULL)->msg_rwait)

#endif	/* _ALL_SOURCE */

#endif	/* _H_MSG */




























































